@@ -115,6 +115,25 @@ MVMInstance * MVM_vm_create_instance(void) {
     uv_once(&key_once, make_uv_key);
 #endif
 
+#ifdef HAVE_TELEMEH
+    if (getenv("MVM_TELEMETRY_LOG")) {
+        char path[256];
+        FILE *fp;
+        snprintf(path, 255, "%s.%d", getenv("MVM_TELEMETRY_LOG"),
+#ifdef _WIN32
+             _getpid()
+#else
+             getpid()
+#endif
+             );
+        fp = MVM_platform_fopen(path, "w");
+        if (fp) {
+            MVM_telemetry_init(fp);
+            MVM_telemetry_interval_start(0, "moarvm startup");
+        }
+    }
+#endif
+
     /* Set up instance data structure. */
     instance = MVM_calloc(1, sizeof(MVMInstance));
 
@@ -309,7 +328,7 @@ MVMInstance * MVM_vm_create_instance(void) {
 
     /* JIT environment/logging setup. */
     jit_disable = getenv("MVM_JIT_DISABLE");
-    if (!jit_disable || !jit_disable[0])
+    if (MVM_jit_support() && (!jit_disable || !jit_disable[0]))
         instance->jit_enabled = 1;
 
     jit_expr_enable = getenv("MVM_JIT_EXPR_ENABLE");
@@ -603,6 +622,13 @@ void MVM_vm_exit(MVMInstance *instance) {
         fclose(instance->dynvar_log_fh);
     }
 
+#ifdef HAVE_TELEMEH
+    if (getenv("MVM_TELEMETRY_LOG")) {
+        MVM_telemetry_interval_stop(0, 1, "moarvm teardown");
+        MVM_telemetry_finish();
+    }
+#endif
+
     /* And, we're done. */
     exit(0);
 }
@@ -761,6 +787,13 @@ void MVM_vm_destroy_instance(MVMInstance *instance) {
      * (e.g., valgrind, heaptrack) don't support mimalloc well. */
     mi_collect(true);
 #endif
+
+#ifdef HAVE_TELEMEH
+    if (getenv("MVM_TELEMETRY_LOG")) {
+        MVM_telemetry_interval_stop(0, 1, "moarvm teardown");
+        MVM_telemetry_finish();
+    }
+#endif
 }
 
 void MVM_vm_set_clargs(MVMInstance *instance, int argc, char **argv) {
@@ -781,7 +814,7 @@ void MVM_vm_event_subscription_configure(MVMThreadContext *tc, MVMObject *queue,
     MVMString *speshoverviewevent;
     MVMString *startup_time;
 
-    MVMROOT2(tc, queue, config, {
+    MVMROOT2(tc, queue, config) {
         if (!IS_CONCRETE(config)) {
             MVM_exception_throw_adhoc(tc, "vmeventsubscribe requires a concrete configuration hash (got a %s type object)", MVM_6model_get_debug_name(tc, config));
         }
@@ -797,12 +830,12 @@ void MVM_vm_event_subscription_configure(MVMThreadContext *tc, MVMObject *queue,
         }
 
         gcevent = MVM_string_utf8_decode(tc, tc->instance->VMString, "gcevent", 7);
-        MVMROOT(tc, gcevent, {
+        MVMROOT(tc, gcevent) {
             speshoverviewevent = MVM_string_utf8_decode(tc, tc->instance->VMString, "speshoverviewevent", 18);
-            MVMROOT(tc, speshoverviewevent, {
+            MVMROOT(tc, speshoverviewevent) {
                 startup_time = MVM_string_utf8_decode(tc, tc->instance->VMString, "startup_time", 12);
-            });
-        });
+            }
+        }
 
         if (MVM_repr_exists_key(tc, config, gcevent)) {
             MVMObject *value = MVM_repr_at_key_o(tc, config, gcevent);
@@ -836,10 +869,10 @@ void MVM_vm_event_subscription_configure(MVMThreadContext *tc, MVMObject *queue,
 
         if (MVM_repr_exists_key(tc, config, startup_time)) {
             /* Value is ignored, it will just be overwritten. */
-            MVMObject *value = NULL; 
-            MVMROOT3(tc, gcevent, speshoverviewevent, startup_time, {
+            MVMObject *value = NULL;
+            MVMROOT3(tc, gcevent, speshoverviewevent, startup_time) {
                     value = MVM_repr_box_num(tc, tc->instance->boot_types.BOOTNum, tc->instance->subscriptions.vm_startup_now);
-            });
+            }
 
             if (MVM_is_null(tc, value)) {
                 uv_mutex_unlock(&tc->instance->subscriptions.mutex_event_subscription);
@@ -847,7 +880,7 @@ void MVM_vm_event_subscription_configure(MVMThreadContext *tc, MVMObject *queue,
             }
             MVM_repr_bind_key_o(tc, config, startup_time, value);
         }
-    });
+    }
 
     uv_mutex_unlock(&tc->instance->subscriptions.mutex_event_subscription);
 }
