@@ -47,10 +47,10 @@ static size_t max_typed_nodes(MVMCallsite *cs, MVMSpeshStatsType *types) {
 
 /* Allocates a spesh arg guard tree with the specified amounst of nodes. */
 static MVMSpeshArgGuard * allocate_tree(MVMThreadContext *tc, MVMuint32 total_nodes) {
-    /* Allocate as a single blob of memory from the FSA. */
+    /* Allocate as a single blob of memory. */
     size_t node_size = total_nodes * sizeof(MVMSpeshArgGuardNode);
     size_t size = sizeof(MVMSpeshArgGuard) + node_size;
-    MVMSpeshArgGuard *tree = MVM_fixed_size_alloc(tc, tc->instance->fsa, size);
+    MVMSpeshArgGuard *tree = MVM_malloc(size);
     tree->nodes = (MVMSpeshArgGuardNode *)((char *)tree + sizeof(MVMSpeshArgGuard));
     tree->used_nodes = 0;
     tree->num_nodes = total_nodes;
@@ -68,13 +68,6 @@ static MVMint32 next_type_index(MVMCallsite *cs, MVMint32 cur_obj_arg_idx) {
     return -1;
 }
 
-/* Turns a type index into an argument index. */
-static MVMint32 type_index_to_arg_index(MVMCallsite *cs, MVMint32 type_index) {
-    return type_index < cs->num_pos
-        ? type_index
-        : (cs->num_pos - 1) + 2 * (type_index - (cs->num_pos - 1));
-}
-
 /* Add a new result node and return its index. */
 static MVMuint32 add_result_node(MVMThreadContext *tc, MVMSpeshArgGuard *tree,
         MVMuint32 result_index) {
@@ -89,7 +82,7 @@ static MVMuint32 add_result_node(MVMThreadContext *tc, MVMSpeshArgGuard *tree,
 static MVMuint32 add_load_node(MVMThreadContext *tc, MVMSpeshArgGuard *tree,
         MVMCallsite *cs, MVMuint32 type_index) {
     tree->nodes[tree->used_nodes].op = MVM_SPESH_GUARD_OP_LOAD_ARG;
-    tree->nodes[tree->used_nodes].arg_index = type_index_to_arg_index(cs, type_index);
+    tree->nodes[tree->used_nodes].arg_index = type_index;
     tree->nodes[tree->used_nodes].yes = 0;
     tree->nodes[tree->used_nodes].no = 0;
     return tree->used_nodes++;
@@ -476,8 +469,7 @@ MVMint32 MVM_spesh_arg_guard_run_types(MVMThreadContext *tc, MVMSpeshArgGuard *a
 /* Evaluates the argument guards. Returns >= 0 if there is a matching spesh
  * candidate, or -1 if there is not. */
 MVMint32 MVM_spesh_arg_guard_run(MVMThreadContext *tc, MVMSpeshArgGuard *ag,
-                                 MVMCallsite *cs, MVMRegister *args,
-                                 MVMint32 *certain) {
+                                 MVMArgs args, MVMint32 *certain) {
     MVMuint32 current_node = 0;
     MVMObject *test = NULL;
     if (!ag)
@@ -486,10 +478,10 @@ MVMint32 MVM_spesh_arg_guard_run(MVMThreadContext *tc, MVMSpeshArgGuard *ag,
         MVMSpeshArgGuardNode *agn = &(ag->nodes[current_node]);
         switch (agn->op) {
             case MVM_SPESH_GUARD_OP_CALLSITE:
-                current_node = agn->cs == cs ? agn->yes : agn->no;
+                current_node = agn->cs == args.callsite ? agn->yes : agn->no;
                 break;
             case MVM_SPESH_GUARD_OP_LOAD_ARG:
-                test = args[agn->arg_index].o;
+                test = args.source[args.map[agn->arg_index]].o;
                 current_node = agn->yes;
                 break;
             case MVM_SPESH_GUARD_OP_STABLE_CONC:
@@ -518,8 +510,13 @@ MVMint32 MVM_spesh_arg_guard_run(MVMThreadContext *tc, MVMSpeshArgGuard *ag,
                 break;
             case MVM_SPESH_GUARD_OP_RESULT:
                 return agn->result;
+            default:
+                MVM_oops(tc, "Unknown spesh arg guard opcode %d reached", agn->op);
         }
-    } while (current_node != 0);
+    } while (current_node != 0 && current_node < ag->num_nodes);
+    if (current_node >= ag->num_nodes) {
+        MVM_oops(tc, "Spesh arg guard program jumped out of bounds: index %u is above %u", current_node, ag->num_nodes);
+    }
     return -1;
 }
 
@@ -638,12 +635,10 @@ void MVM_spesh_arg_guard_gc_describe(MVMThreadContext *tc, MVMHeapSnapshotState
  * to zero, the memory is freed immediately. */
 void MVM_spesh_arg_guard_destroy(MVMThreadContext *tc, MVMSpeshArgGuard *ag, MVMuint32 safe) {
     if (ag) {
-        size_t total_size = sizeof(MVMSpeshArgGuard) +
-            ag->num_nodes * sizeof(MVMSpeshArgGuardNode);
         if (safe)
-            MVM_fixed_size_free_at_safepoint(tc, tc->instance->fsa, total_size, ag);
+            MVM_free_at_safepoint(tc, ag);
         else
-            MVM_fixed_size_free(tc, tc->instance->fsa, total_size, ag);
+            MVM_free(ag);
     }
 }
 
