@@ -21,8 +21,8 @@ typedef struct {
     /* Fixups we need to do by basic block. */
     MVMint32    *fixup_locations;
     MVMSpeshBB **fixup_bbs;
-    MVMint32     num_fixups;
-    MVMint32     alloc_fixups;
+    MVMuint32    num_fixups;
+    MVMuint32    alloc_fixups;
 
     /* Copied frame handlers (which we'll update offsets of). */
     MVMFrameHandler *handlers;
@@ -31,6 +31,8 @@ typedef struct {
      * inline from this bytecode in the future. */
     MVM_VECTOR_DECL(MVMint32, deopt_usage_info);
 
+    MVM_VECTOR_DECL(MVMint32, deopt_synth_addrs);
+
     /* Working deopt users state (so we can allocate it once and re-use it). */
     AllDeoptUsers all_deopt_users;
 } SpeshWriterState;
@@ -47,6 +49,11 @@ static void write_int64(SpeshWriterState *ws, MVMuint64 value) {
     memcpy(ws->bytecode + ws->bytecode_pos, &value, 8);
     ws->bytecode_pos += 8;
 }
+static void write_uint64(SpeshWriterState *ws, MVMuint64 value) {
+    ensure_space(ws, 8);
+    memcpy(ws->bytecode + ws->bytecode_pos, &value, 8);
+    ws->bytecode_pos += 8;
+}
 static void write_int32(SpeshWriterState *ws, MVMuint32 value) {
     ensure_space(ws, 4);
     memcpy(ws->bytecode + ws->bytecode_pos, &value, 4);
@@ -87,7 +94,7 @@ static void collect_deopt_users(MVMThreadContext *tc, MVMSpeshGraph *g, MVMSpesh
         MVMSpeshIns *ins = users->user;
         if (ins->info->opcode == MVM_SSA_PHI) {
             MVMint32 seen = 0;
-            MVMint32 i;
+            MVMuint32 i;
             for (i = 0; i < MVM_VECTOR_ELEMS(all_deopt_users->seen_phis); i++) {
                 if (all_deopt_users->seen_phis[i] == ins) {
                     seen = 1;
@@ -130,6 +137,7 @@ static void write_instructions(MVMThreadContext *tc, MVMSpeshGraph *g, SpeshWrit
             case MVM_SPESH_ANN_DEOPT_ONE_INS:
             case MVM_SPESH_ANN_DEOPT_ALL_INS:
             case MVM_SPESH_ANN_DEOPT_INLINE:
+            case MVM_SPESH_ANN_DEOPT_SYNTH:
                 has_deopts = 1;
                 break;
             case MVM_SPESH_ANN_INLINE_START:
@@ -139,7 +147,8 @@ static void write_instructions(MVMThreadContext *tc, MVMSpeshGraph *g, SpeshWrit
                 g->inlines[ann->data.inline_idx].end = ws->bytecode_pos;
                 break;
             case MVM_SPESH_ANN_DEOPT_OSR:
-                g->deopt_addrs[2 * ann->data.deopt_idx + 1] = ws->bytecode_pos;
+            case MVM_SPESH_ANN_DEOPT_PRE_INS:
+                g->deopt_addrs[2 * ann->data.deopt_idx + 1] = ws->bytecode_pos << 1 | 1;
                 break;
             }
             ann = ann->next;
@@ -184,6 +193,45 @@ static void write_instructions(MVMThreadContext *tc, MVMSpeshGraph *g, SpeshWrit
             else {
                 /* Core op. */
                 write_int16(ws, ins->info->opcode);
+
+                /* sp_resumption needs entries making in the spesh resume init
+                 * table. */
+                if (ins->info->opcode == MVM_OP_sp_resumption) {
+                    /* The state register is simply the register number. */
+                    MVMSpeshResumeInit *ri = &(g->resume_inits[ins->operands[1].lit_ui16]);
+                    ri->state_register = ins->operands[0].reg.orig;
+
+                    /* For the resume init registers, we only have registers
+                     * for args and temps, but not constants. We need to build
+                     * a table that has those register numbers in the correct
+                     * place. */
+                    MVMDispProgramResumption *dpr = &(ri->dp->resumptions[ri->res_idx]);
+                    MVMuint16 map_size = dpr->init_callsite->flag_count;
+                    ri->init_registers = MVM_malloc(map_size * sizeof(MVMuint16));
+                    MVMuint16 num_init_registers = ins->operands[2].lit_ui16;
+                    if (dpr->init_values) {
+                        MVMuint16 cur_reg = 0;
+                        for (i = 0; i < map_size; i++) {
+                            switch (dpr->init_values[i].source) {
+                                case MVM_DISP_RESUME_INIT_ARG:
+                                case MVM_DISP_RESUME_INIT_TEMP:
+                                    ri->init_registers[i] = ins->operands[3 + cur_reg].reg.orig;
+                                    cur_reg++;
+                                    break;
+                                default:
+                                    ri->init_registers[i] = 0xFFFF;
+                                    break;
+                            }
+                        }
+                    }
+                    else {
+                        /* It's just the initial arguments to the dispatch, so
+                         * should match directly with the op registers. */
+                        assert(map_size == num_init_registers);
+                        for (i = 0; i < num_init_registers; i++)
+                            ri->init_registers[i] = ins->operands[3 + i].reg.orig;
+                    }
+                }
             }
 
             /* Write out operands. */
@@ -209,6 +257,9 @@ static void write_instructions(MVMThreadContext *tc, MVMSpeshGraph *g, SpeshWrit
                     case MVM_operand_int16:
                         write_int16(ws, ins->operands[i].lit_i16);
                         break;
+                    case MVM_operand_uint16:
+                        write_int16(ws, ins->operands[i].lit_i16);
+                        break;
                     case MVM_operand_int32:
                         write_int32(ws, ins->operands[i].lit_i32);
                         break;
@@ -218,6 +269,9 @@ static void write_instructions(MVMThreadContext *tc, MVMSpeshGraph *g, SpeshWrit
                     case MVM_operand_int64:
                         write_int64(ws, ins->operands[i].lit_i64);
                         break;
+                    case MVM_operand_uint64:
+                        write_uint64(ws, ins->operands[i].lit_ui64);
+                        break;
                     case MVM_operand_num32:
                         write_num32(ws, ins->operands[i].lit_n32);
                         break;
@@ -234,7 +288,7 @@ static void write_instructions(MVMThreadContext *tc, MVMSpeshGraph *g, SpeshWrit
                         write_int32(ws, ins->operands[i].lit_str_idx);
                         break;
                     case MVM_operand_ins: {
-                        MVMint32 bb_idx = ins->operands[i].ins_bb->idx;
+                        MVMuint32 bb_idx = ins->operands[i].ins_bb->idx;
                         MVMint32 offset;
                         if (bb_idx >= g->num_bbs)
                             MVM_panic(1, "Spesh codegen: out of range BB index %d", bb_idx);
@@ -277,17 +331,57 @@ static void write_instructions(MVMThreadContext *tc, MVMSpeshGraph *g, SpeshWrit
 
         /* If there were deopt point annotations, update table. */
         if (has_deopts) {
+#ifndef NDEBUG
+            MVMint32 seen_deopt_idx = 0;
+            MVMint32 deopt_idx;
+            switch (ins->info->opcode) {
+            case MVM_OP_sp_guard:
+            case MVM_OP_sp_guardconc:
+            case MVM_OP_sp_guardtype:
+            case MVM_OP_sp_guardobj:
+            case MVM_OP_sp_guardnotobj:
+            case MVM_OP_sp_guardhll:
+            case MVM_OP_sp_rebless:
+                deopt_idx = ins->operands[3].lit_ui32;
+                break;
+            case MVM_OP_sp_guardsf:
+            case MVM_OP_sp_guardsfouter:
+            case MVM_OP_sp_guardjustconc:
+            case MVM_OP_sp_guardjusttype:
+            case MVM_OP_sp_guardnonzero:
+                deopt_idx = ins->operands[2].lit_ui32;
+                break;
+            default:
+                deopt_idx = -1;
+                break;
+            }
+#endif
             ann = ins->annotations;
             while (ann) {
                 switch (ann->type) {
+                case MVM_SPESH_ANN_DEOPT_SYNTH:
+                    MVM_VECTOR_PUSH(ws->deopt_synth_addrs, ann->data.deopt_idx);
+                    MVM_VECTOR_PUSH(ws->deopt_synth_addrs, ws->bytecode_pos);
+                    break;
                 case MVM_SPESH_ANN_DEOPT_ONE_INS:
                 case MVM_SPESH_ANN_DEOPT_ALL_INS:
                 case MVM_SPESH_ANN_DEOPT_INLINE:
-                    g->deopt_addrs[2 * ann->data.deopt_idx + 1] = ws->bytecode_pos;;
+                    g->deopt_addrs[2 * ann->data.deopt_idx + 1] = ws->bytecode_pos << 1;
+#ifndef NDEBUG
+                    if (deopt_idx == ann->data.deopt_idx)
+                        seen_deopt_idx = 1;
+#endif
                     break;
+#ifndef NDEBUG
+                case MVM_SPESH_ANN_DEOPT_PRE_INS:
+                    if (deopt_idx == ann->data.deopt_idx)
+                        seen_deopt_idx = 1;
+                break;
+#endif
                 }
                 ann = ann->next;
             }
+            assert(deopt_idx < 0 || seen_deopt_idx);
         }
 
         ins = ins->next;
@@ -298,7 +392,7 @@ static void write_instructions(MVMThreadContext *tc, MVMSpeshGraph *g, SpeshWrit
 MVMSpeshCode * MVM_spesh_codegen(MVMThreadContext *tc, MVMSpeshGraph *g) {
     MVMSpeshCode *res;
     MVMSpeshBB   *bb;
-    MVMint32      i, hanlen;
+    MVMuint32     i, hanlen;
 
     /* Initialize writer state. */
     SpeshWriterState *ws     = MVM_malloc(sizeof(SpeshWriterState));
@@ -315,6 +409,7 @@ MVMSpeshCode * MVM_spesh_codegen(MVMThreadContext *tc, MVMSpeshGraph *g) {
     MVM_VECTOR_INIT(ws->deopt_usage_info, 0);
     MVM_VECTOR_INIT(ws->all_deopt_users.idxs, 0);
     MVM_VECTOR_INIT(ws->all_deopt_users.seen_phis, 0);
+    MVM_VECTOR_INIT(ws->deopt_synth_addrs, 0);
 
     /* Create copy of handlers, and -1 all offsets so we can catch missing
      * updates. */
@@ -352,8 +447,8 @@ MVMSpeshCode * MVM_spesh_codegen(MVMThreadContext *tc, MVMSpeshGraph *g) {
 
     /* Fixup labels we were too early for. */
     for (i = 0; i < ws->num_fixups; i++)
-        *((MVMuint32 *)(ws->bytecode + ws->fixup_locations[i])) =
-            ws->bb_offsets[ws->fixup_bbs[i]->idx];
+        memcpy((ws->bytecode + ws->fixup_locations[i]),
+               ws->bb_offsets + ws->fixup_bbs[i]->idx, sizeof(MVMuint32));
 
     /* Ensure all handlers that are reachable got fixed up. */
     for (i = 0; i < g->num_handlers; i++) {
@@ -362,9 +457,9 @@ MVMSpeshCode * MVM_spesh_codegen(MVMThreadContext *tc, MVMSpeshGraph *g) {
             ws->handlers[i].end_offset = -1;
             ws->handlers[i].goto_offset = -1;
         }
-        else if (ws->handlers[i].start_offset == -1 ||
-                 ws->handlers[i].end_offset   == -1 ||
-                 ws->handlers[i].goto_offset  == -1) {
+        else if (ws->handlers[i].start_offset == (MVMuint32)-1 ||
+                 ws->handlers[i].end_offset   == (MVMuint32)-1 ||
+                 ws->handlers[i].goto_offset  == (MVMuint32)-1) {
             MVM_oops(tc, "Spesh: failed to fix up handler %d in %s (%d, %d, %d)",
                 i,
                 MVM_string_utf8_maybe_encode_C_string(tc, g->sf->body.name),
@@ -381,7 +476,7 @@ MVMSpeshCode * MVM_spesh_codegen(MVMThreadContext *tc, MVMSpeshGraph *g) {
             g->inlines[i].end = -1;
         }
         else {
-            if (g->inlines[i].start == -1 || g->inlines[i].end == -1)
+            if (g->inlines[i].start == (MVMuint32)-1 || g->inlines[i].end == (MVMuint32)-1)
                 MVM_oops(tc, "Spesh: failed to fix up inline %d (%s) %d %d",
                     i,
                     MVM_string_utf8_maybe_encode_C_string(tc, g->inlines[i].sf->body.name),
@@ -401,6 +496,8 @@ MVMSpeshCode * MVM_spesh_codegen(MVMThreadContext *tc, MVMSpeshGraph *g) {
     res->bytecode_size    = ws->bytecode_pos;
     res->handlers         = ws->handlers;
     res->deopt_usage_info = ws->deopt_usage_info;
+    res->deopt_synths     = ws->deopt_synth_addrs;
+    res->num_deopt_synths = ws->deopt_synth_addrs_num / 2; /* 2 values per entry */
 
     /* Cleanup. */
     MVM_free(ws->bb_offsets);
